<?php
namespace FiloBlu\ExpiredSessionRedirect\Observer;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Magento\Framework\Stdlib\Cookie\PhpCookieManager;
use Psr\Log\LoggerInterface;

class SetCookie implements \Magento\Framework\Event\ObserverInterface
{
    const COOKIE_NAME = 'lcs';
    const EXPIRED_SESSION_REDIRECT_ENABLE ='filoblu_expiredsessionredirect_section/expiredsessionredirect_group/expiredsessionredirect_enable';
    const EXPIRED_SESSION_REDIRECT_URL  = 'filoblu_expiredsessionredirect_section/expiredsessionredirect_group/expiredsessionredirect_url';
    /**
     * @var PhpCookieManager
     */
    private $cookieManager;
    /**
     * @var CookieMetadataFactory
     */
    private $cookieMetadataFactory;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;


    /**
     * SetCookie constructor.
     * @param PhpCookieManager $cookieManager
     * @param CookieMetadataFactory $cookieMetadataFactory
     * @param LoggerInterface $logger
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        PhpCookieManager $cookieManager,
        CookieMetadataFactory $cookieMetadataFactory,
        LoggerInterface $logger,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->cookieManager = $cookieManager;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        try {
            if ($this->scopeConfig->getValue(self::EXPIRED_SESSION_REDIRECT_ENABLE, \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE) ?? false) {
                if ($customerSess = $observer->getCustomerSession()) {
                    $customer = $customerSess->getCustomer();
                    if ($customer && $customer->getId()) {
                        $customerHash = password_hash($customer->getId(), PASSWORD_DEFAULT);
                        if (!password_verify($this->cookieManager->getCookie(self::COOKIE_NAME), $customerHash)) {
                            $this->cookieManager->deleteCookie(self::COOKIE_NAME);
                            $metadata = $this->cookieMetadataFactory->createPublicCookieMetadata()->setPath('/');
                            $this->cookieManager->setPublicCookie(self::COOKIE_NAME, $customerHash, $metadata);
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        } catch (\Throwable $t) {
            $this->logger->critical($t->getMessage());
        }
    }
}
